@@ -9,10 +9,9 @@ import sys
 import tempfile
 from multiprocessing.pool import ThreadPool
 from time import sleep
-from typing import IO, TYPE_CHECKING, Any, Callable, Iterable
+from typing import IO, TYPE_CHECKING, Any, Callable, Iterable, TypeVar
 
 import pyalpm
-from typing_extensions import NotRequired, TypedDict
 
 from .args import parse_args
 from .config import PikaurConfig
@@ -21,11 +20,20 @@ from .pprint import ColorsHighlight, bold_line, color_line, print_error, print_s
 
 if TYPE_CHECKING:
     # pylint: disable=cyclic-import
+    from typing_extensions import NotRequired, TypedDict
+
     from .aur import AURPackageInfo
 
+    class SpawnArgs(TypedDict):
+        stdout: NotRequired['IOStream']
+        stderr: NotRequired['IOStream']
+        cwd: NotRequired[str]
+        env: NotRequired[dict[str, str]]
+
 
 DEFAULT_INPUT_ENCODING = 'utf-8'
 PIPE = subprocess.PIPE
+IOStream = IO[bytes] | int | None
 
 
 class ComparableType:
@@ -33,10 +41,10 @@ class ComparableType:
     __ignore_in_eq__: tuple[str, ...] = ()
 
     __hash__ = object.__hash__
-    __compare_stack__: list[Any] | None = None
+    __compare_stack__: list['ComparableType'] | None = None
 
     @property
-    def public_vars(self) -> dict[str, Any]:
+    def public_values(self) -> dict[str, Any]:
         return {
             var: val for var, val in vars(self).items()
             if not var.startswith('__')
@@ -50,15 +58,15 @@ class ComparableType:
         elif other in self.__compare_stack__:
             return super().__eq__(other)
         self.__compare_stack__.append(other)
-        self_vars = {}
-        self_vars.update(self.public_vars)
-        other_vars = {}
-        other_vars.update(other.public_vars)
-        for var_dict in (self_vars, other_vars):
+        self_values = {}
+        self_values.update(self.public_values)
+        others_values = {}
+        others_values.update(other.public_values)
+        for values in (self_values, others_values):
             for skip_prop in self.__ignore_in_eq__:
-                if skip_prop in var_dict:
-                    del var_dict[skip_prop]
-        result = self_vars == other_vars
+                if skip_prop in values:
+                    del values[skip_prop]
+        result = self_values == others_values
         self.__compare_stack__ = None
         return result
 
@@ -66,8 +74,8 @@ class ComparableType:
 class DataType(ComparableType):
 
     @property
-    def __all_annotations__(self) -> dict[str, Any]:
-        annotations: dict[str, Any] = {}
+    def __all_annotations__(self) -> dict[str, type]:
+        annotations: dict[str, type] = {}
         for parent_class in reversed(self.__class__.mro()):
             annotations.update(**getattr(parent_class, '__annotations__', {}))
         return annotations
@@ -153,13 +161,13 @@ def get_sudo_refresh_command() -> list[str]:
     return sudo([pacman_path, '-T'])
 
 
-class InteractiveSpawn(subprocess.Popen):
+class InteractiveSpawn(subprocess.Popen[bytes]):
 
-    stdout_text: str
-    stderr_text: str
+    stdout_text: str | None
+    stderr_text: str | None
 
-    def communicate(  # pylint: disable=arguments-renamed
-            self, com_input: str | None = None, timeout: float | None = None
+    def communicate(
+            self, com_input: bytes | None = None, timeout: float | None = None
     ) -> tuple[bytes, bytes]:
         if parse_args().print_commands:
             if self.args != get_sudo_refresh_command():
@@ -189,21 +197,14 @@ class InteractiveSpawn(subprocess.Popen):
         self.communicate()
 
 
-class SpawnArgs(TypedDict):
-    stdout: NotRequired[IO[Any] | int | None]
-    stderr: NotRequired[IO[Any] | int | None]
-    cwd: NotRequired[str]
-    env: NotRequired[dict[str, str]]
-
-
 def interactive_spawn(
         cmd: list[str],
-        stdout: IO[Any] | int | None = None,
-        stderr: IO[Any] | int | None = None,
+        stdout: IOStream = None,
+        stderr: IOStream = None,
         cwd: str | None = None,
         env: dict[str, str] | None = None,
 ) -> InteractiveSpawn:
-    kwargs: SpawnArgs = {}
+    kwargs: 'SpawnArgs' = {}
     if stdout:
         kwargs['stdout'] = stdout
     if stderr:
@@ -345,7 +346,10 @@ def remove_dir(dir_path: str) -> None:
         interactive_spawn(sudo(['rm', '-rf', dir_path]))
 
 
-def get_chunks(iterable: Iterable[Any], chunk_size: int) -> Iterable[list[Any]]:
+ChunkT = TypeVar('ChunkT')
+
+
+def get_chunks(iterable: Iterable[ChunkT], chunk_size: int) -> Iterable[list[ChunkT]]:
     if chunk_size < 1:
         raise ValueError("`chunk_size` can't be smaller than 1.")
     result = []
@@ -392,14 +396,17 @@ def sudo_loop(once: bool = False) -> None:
         sleep(sudo_loop_interval)
 
 
-def run_with_sudo_loop(function: Callable) -> Any | None:
+SudoLoopResultT = TypeVar('SudoLoopResultT')
+
+
+def run_with_sudo_loop(function: Callable[..., SudoLoopResultT]) -> SudoLoopResultT | None:
     sudo_loop(once=True)
     with ThreadPool(processes=2) as pool:
         main_thread = pool.apply_async(function, ())
         pool.apply_async(sudo_loop)
         pool.close()
         catched_exc = None
-        result: Any = None
+        result: SudoLoopResultT | None = None
         try:
             result = main_thread.get()
         except Exception as exc:
