@@ -3,15 +3,16 @@
 """Licensed under GPLv3, see https://www.gnu.org/licenses/"""
 
 import atexit
-import codecs
 import io
 import os
 import readline
 import shutil
 import signal
 import sys
+import traceback
 from argparse import ArgumentError
 from contextlib import AbstractContextManager
+from types import TracebackType
 from typing import Any, Callable
 
 import pyalpm
@@ -46,7 +47,7 @@ from .pkg_cache_cli import cli_clean_packages_cache
 from .pprint import create_debug_logger, print_error, print_stderr, print_warning
 from .print_department import print_version
 from .prompt import NotANumberInputError, get_multiple_numbers_input
-from .search_cli import cli_search_packages
+from .search_cli import cli_search_packages, search_packages
 from .updates import print_upgradeable
 from .urllib import ProxyInitSocks5Error, init_proxy
 
@@ -66,15 +67,19 @@ init_readline()
 _debug = create_debug_logger('main')
 
 
-class OutputEncodingWrapper():
+class OutputEncodingWrapper(AbstractContextManager[None]):
 
     original_stderr: int
     original_stdout: int
 
     def __enter__(self) -> None:
         for attr in ('stdout', 'stderr'):
-            _debug(f'Setting {attr} to unicode...')
+            _debug(f'Setting {attr} to {DEFAULT_INPUT_ENCODING}...', lock=False)
             real_stream = getattr(sys, attr)
+            if real_stream.encoding == DEFAULT_INPUT_ENCODING:
+                _debug('already set - nothing to do', lock=False)
+                continue
+            real_stream.flush()
             try:
                 setattr(
                     self, f'original_{attr}',
@@ -82,26 +87,45 @@ class OutputEncodingWrapper():
                 )
                 setattr(
                     sys, attr,
-                    codecs.open(
+                    io.open(
                         real_stream.fileno(),
-                        mode='w', buffering=0, encoding=DEFAULT_INPUT_ENCODING
+                        mode='w',
+                        encoding=DEFAULT_INPUT_ENCODING,
+                        closefd=False,
                     )
                 )
-            except io.UnsupportedOperation:
-                pass
-            else:
-                getattr(sys, attr).buffer = real_stream.buffer
+            except io.UnsupportedOperation as exc:
+                _debug(f"Can't set {attr} to {DEFAULT_INPUT_ENCODING}:\n{exc}", lock=False)
 
-    def __exit__(self, *_exc_details: Any) -> None:
-        for attr in ('stdout', 'stderr'):
-            _debug(f'Restoring {attr}...', lock=False)
-            stream = getattr(sys, attr)
-            orig_stream = getattr(self, f'original_{attr}')
-            stream.close()
-            setattr(
-                sys, attr,
-                orig_stream
-            )
+    def __exit__(
+            self,
+            exc_class: type | None, exc_instance: BaseException | None, exc_tb: TracebackType | None
+    ) -> None:
+        try:
+            # @TODO: replace all SysExit-s to SystemExit-s eventually :3
+            if exc_instance and exc_class and (exc_class not in (SysExit, SystemExit, )):
+                # handling exception in context manager's __exit__ is not recommended
+                # but otherwise stderr would be closed before exception is printed...
+                if exc_tb:
+                    print_stderr(''.join(traceback.format_tb(exc_tb)), lock=False)
+                print_stderr(f"{exc_class.__name__}: {exc_instance}", lock=False)
+                sys.exit(121)
+        finally:
+            for attr in ('stdout', 'stderr'):
+                _debug(f'Restoring {attr}...', lock=False)
+                stream = getattr(sys, attr)
+                orig_stream = getattr(self, f'original_{attr}', None)
+                if orig_stream in (None, stream, ):
+                    _debug('nothing to do', lock=False)
+                    continue
+                stream.flush()
+                setattr(
+                    sys, attr,
+                    orig_stream
+                )
+                _debug(f'{attr} restored', lock=False)
+                stream.close()
+                _debug(f'closed old {attr} stream', lock=False)
 
 
 def cli_print_upgradeable() -> None:
@@ -118,9 +142,13 @@ def cli_pkgbuild() -> None:
 
 def cli_print_version() -> None:
     args = parse_args()
-    pacman_version = spawn(
+    proc = spawn(
         [PikaurConfig().misc.PacmanPath.get_str(), '--version', ],
-    ).stdout_text.splitlines()[1].strip(' .-')
+    )
+    if proc.stdout_text:
+        pacman_version = proc.stdout_text.splitlines()[1].strip(' .-')
+    else:
+        pacman_version = "N/A"
     print_version(
         pacman_version=pacman_version, pyalpm_version=pyalpm.version(),
         quiet=args.quiet
@@ -128,7 +156,7 @@ def cli_print_version() -> None:
 
 
 def cli_dynamic_select() -> None:  # pragma: no cover
-    packages = cli_search_packages(enumerated=True)
+    packages = search_packages(enumerated=True)
     if not packages:
         raise SysExit(1)
 
@@ -176,7 +204,7 @@ def cli_entry_point() -> None:  # pylint: disable=too-many-statements
     # specified both operations, like `pikaur -QS smth`
 
     args = parse_args()
-    pikaur_operation: Callable | None = None
+    pikaur_operation: Callable[[], None] | None = None
     require_sudo = False
 
     if args.help:
@@ -318,7 +346,7 @@ class EmptyWrapper:
 
 
 def main(embed: bool = False) -> None:
-    wrapper: type[AbstractContextManager] = OutputEncodingWrapper
+    wrapper: type[AbstractContextManager[None]] = OutputEncodingWrapper
     if embed:
         wrapper = EmptyWrapper
     with wrapper():
