@@ -20,8 +20,13 @@ def color_enabled() -> bool:
     args = ARGS
     if args.color == 'never':
         return False
-    if args.color == 'always' or (sys.stderr.isatty() and sys.stdout.isatty()):
+    if args.color == 'always':
         return True
+    try:
+        if (sys.stderr.isatty() and sys.stdout.isatty()):
+            return True
+    except Exception:
+        return False
     return False
 
 
@@ -233,7 +238,7 @@ def create_debug_logger(module_name: str, lock: bool | None = None) -> t.Callabl
     parent_lock = lock
 
     def debug(msg: Any, lock: bool | None = None) -> None:
-        lock = lock or parent_lock
+        lock = lock if (lock is not None) else parent_lock
         msg = f"{color_line(module_name, color)}: {str(msg)}"
         if lock is not None:
             print_debug(msg, lock=lock)
